@@ -15,6 +15,7 @@
 #include "reservationhelper.h"
 #include "reservationmanager.h"
 #include "publictransport.h"
+#include "publictransportmatcher.h"
 #include "transfer.h"
 #include "transfermanager.h"
 
@@ -26,8 +27,14 @@
 #include <KItinerary/LocationUtil>
 #include <KItinerary/SortUtil>
 #include <KItinerary/Reservation>
+#include <KItinerary/Ticket>
 #include <KItinerary/TrainTrip>
 
+#include <KPublicTransport/Platform>
+#include <KPublicTransport/PlatformLayout>
+#include <KPublicTransport/Stopover>
+#include <KPublicTransport/Vehicle>
+
 #include <QJSEngine>
 #include <QJSValue>
 
@@ -76,6 +83,7 @@ void TimelineDelegateController::setReservationManager(QObject *resMgr)
     Q_EMIT departureChanged();
     Q_EMIT arrivalChanged();
     Q_EMIT previousLocationChanged();
+    Q_EMIT layoutChanged();
 
     connect(m_resMgr, &ReservationManager::batchChanged, this, &TimelineDelegateController::batchChanged);
     connect(m_resMgr, &ReservationManager::batchContentChanged, this, &TimelineDelegateController::batchChanged);
@@ -101,6 +109,7 @@ void TimelineDelegateController::setLiveDataManager(QObject* liveDataMgr)
     Q_EMIT setupChanged();
     Q_EMIT departureChanged();
     Q_EMIT arrivalChanged();
+    Q_EMIT layoutChanged();
 
     connect(m_liveDataMgr, &LiveDataManager::arrivalUpdated, this, &TimelineDelegateController::checkForUpdate);
     connect(m_liveDataMgr, &LiveDataManager::departureUpdated, this, &TimelineDelegateController::checkForUpdate);
@@ -146,6 +155,7 @@ void TimelineDelegateController::setDocumentManager(QObject *documentMgr)
 
     m_documentMgr = qobject_cast<DocumentManager*>(documentMgr);
     Q_EMIT setupChanged();
+    Q_EMIT contentChanged();
 }
 
 QString TimelineDelegateController::batchId() const
@@ -518,7 +528,7 @@ void TimelineDelegateController::applyJourney(const QVariant &journey, bool incl
         }
 
         // same type -> update the existing one
-        if (PublicTransport::isSameMode(oldRes, section)) {
+        if (PublicTransportMatcher::isSameMode(oldRes, section)) {
             const auto resIds = m_resMgr->reservationsForBatch(*it);
             for (const auto &resId : resIds) {
                 auto res = m_resMgr->reservation(resId);
@@ -601,11 +611,50 @@ static void mapArgumentsForLocation(QJSValue &args, const QVariant &location, QJ
     args.setProperty(QStringLiteral("coordinate"), engine->toScriptValue(QPointF(geo.longitude(), geo.latitude())));
 }
 
-static void mapArgumentsForPt(QJSValue &args, QLatin1String prefix, const KPublicTransport::Stopover &stop)
+struct CoachData {
+    QString coachName;
+    KPublicTransport::VehicleSection::Class coachClass = KPublicTransport::VehicleSection::UnknownClass;
+};
+
+static CoachData coachDataForReservation(const QVariant &res)
+{
+    if (!JsonLd::isA<TrainReservation>(res)) {
+        return {};
+    }
+
+    const auto trainRes = res.value<TrainReservation>();
+    const auto seat = trainRes.reservedTicket().value<Ticket>().ticketedSeat();
+
+    CoachData data;
+    data.coachName = seat.seatSection();
+    if (seat.seatingType() == QLatin1String("1")) {
+        data.coachClass = KPublicTransport::VehicleSection::FirstClass;
+    } else if (seat.seatingType() == QLatin1String("2")) {
+        data.coachClass = KPublicTransport::VehicleSection::SecondClass;
+    }
+    return data;
+}
+
+static QString platformSectionsForCoachData(const KPublicTransport::Stopover &stop, const CoachData &coach)
+{
+    KPublicTransport::PlatformLayout layouter;
+    if (!coach.coachName.isEmpty()) {
+        return layouter.sectionsForVehicleSection(stop, coach.coachName);
+    } else if (coach.coachClass != KPublicTransport::VehicleSection::UnknownClass) {
+        return layouter.sectionsForClass(stop, coach.coachClass);
+    } else {
+        return layouter.sectionsForVehicle(stop);
+    }
+
+    return {};
+}
+
+static void mapArgumentsForPt(QJSValue &args, QLatin1String prefix, const KPublicTransport::Stopover &stop, const CoachData &coach)
 {
     const auto platformName = stop.hasExpectedPlatform() ? stop.expectedPlatform() : stop.scheduledPlatform();
     if (!platformName.isEmpty()) {
-        args.setProperty(prefix + QLatin1String("PlatformName"), platformName);
+        const auto sections = platformSectionsForCoachData(stop, coach);
+        args.setProperty(prefix + QLatin1String("PlatformName"), sections.isEmpty() ? platformName : (platformName + QLatin1Char(' ') + sections));
     }
 
     if (stop.route().line().mode() != KPublicTransport::Line::Unknown) {
@@ -666,7 +715,7 @@ QJSValue TimelineDelegateController::arrivalMapArguments() const
     // arrival location
     mapArrivalArgumesForRes(args, res);
     const auto arr = arrival();
-    mapArgumentsForPt(args, QLatin1String("arrival"), arr);
+    mapArgumentsForPt(args, QLatin1String("arrival"), arr, coachDataForReservation(res));
 
     // arrival time
     auto arrTime = arr.hasExpectedArrivalTime() ? arr.expectedArrivalTime() : arr.scheduledArrivalTime();
@@ -682,7 +731,7 @@ QJSValue TimelineDelegateController::arrivalMapArguments() const
     const auto transfer = m_transferMgr->transfer(m_batchId, Transfer::After);
     if (transfer.state() == Transfer::Selected) {
         const auto dep = PublicTransport::firstTransportSection(transfer.journey()).departure();
-        mapArgumentsForPt(args, QLatin1String("departure"), dep);
+        mapArgumentsForPt(args, QLatin1String("departure"), dep, {});
         args.setProperty(QStringLiteral("endTime"), engine->toScriptValue(dep.hasExpectedDepartureTime() ? dep.expectedDepartureTime() : dep.scheduledDepartureTime()));
         return args;
     }
@@ -695,7 +744,7 @@ QJSValue TimelineDelegateController::arrivalMapArguments() const
     }
     mapDepartureArgumentsForRes(args, nextRes);
     const auto dep = m_liveDataMgr->departure(nextResId);
-    mapArgumentsForPt(args, QLatin1String("departure"), dep);
+    mapArgumentsForPt(args, QLatin1String("departure"), dep, coachDataForReservation(nextRes));
 
     auto depTime = dep.hasExpectedDepartureTime() ? dep.expectedDepartureTime() : dep.scheduledDepartureTime();
     if (!depTime.isValid()) {
@@ -724,7 +773,7 @@ QJSValue TimelineDelegateController::departureMapArguments() const
     // departure location
     mapDepartureArgumentsForRes(args, res);
     const auto dep = departure();
-    mapArgumentsForPt(args, QLatin1String("departure"), dep);
+    mapArgumentsForPt(args, QLatin1String("departure"), dep, coachDataForReservation(res));
 
     // departure time
     auto depTime = dep.hasExpectedDepartureTime() ? dep.expectedDepartureTime() : dep.scheduledDepartureTime();
@@ -740,7 +789,7 @@ QJSValue TimelineDelegateController::departureMapArguments() const
     const auto transfer = m_transferMgr->transfer(m_batchId, Transfer::Before);
     if (transfer.state() == Transfer::Selected) {
         const auto arr = PublicTransport::lastTransportSection(transfer.journey()).arrival();
-        mapArgumentsForPt(args, QLatin1String("arrival"), arr);
+        mapArgumentsForPt(args, QLatin1String("arrival"), arr, {});
         args.setProperty(QStringLiteral("beginTime"), engine->toScriptValue(arr.hasExpectedArrivalTime() ? arr.expectedArrivalTime() : arr.scheduledArrivalTime()));
         return args;
     }
@@ -753,7 +802,7 @@ QJSValue TimelineDelegateController::departureMapArguments() const
     }
     mapArrivalArgumesForRes(args, prevRes);
     const auto arr = m_liveDataMgr->arrival(prevResId);
-    mapArgumentsForPt(args, QLatin1String("arrival"), arr);
+    mapArgumentsForPt(args, QLatin1String("arrival"), arr, coachDataForReservation(prevRes));
 
     auto arrTime = arr.hasExpectedArrivalTime() ? arr.expectedArrivalTime() : arr.scheduledArrivalTime();
     if (!arrTime.isValid()) {
@@ -798,7 +847,7 @@ QJSValue TimelineDelegateController::mapArguments() const
     auto transfer = m_transferMgr->transfer(m_batchId, Transfer::Before);
     if (transfer.state() == Transfer::Selected) {
         const auto arr = PublicTransport::lastTransportSection(transfer.journey()).arrival();
-        mapArgumentsForPt(args, QLatin1String("arrival"), arr);
+        mapArgumentsForPt(args, QLatin1String("arrival"), arr, {});
         beginDt = std::max(arr.hasExpectedArrivalTime() ? arr.expectedArrivalTime() : arr.scheduledArrivalTime(), beginDt);
     }
 
@@ -813,7 +862,7 @@ QJSValue TimelineDelegateController::mapArguments() const
     transfer = m_transferMgr->transfer(m_batchId, Transfer::After);
     if (transfer.state() == Transfer::Selected) {
         const auto dep = PublicTransport::firstTransportSection(transfer.journey()).departure();
-        mapArgumentsForPt(args, QLatin1String("departure"), dep);
+        mapArgumentsForPt(args, QLatin1String("departure"), dep, {});
         const auto depDt = dep.hasExpectedDepartureTime() ? dep.expectedDepartureTime() : dep.scheduledDepartureTime();
         endDt = endDt.isValid() ? std::min(endDt, depDt) : depDt;
     }
@@ -842,26 +891,6 @@ QJSValue TimelineDelegateController::mapArguments() const
     return args;
 }
 
-int TimelineDelegateController::documentCount() const
-{
-    if (!m_resMgr || m_batchId.isEmpty()) {
-        return 0;
-    }
-
-    QSet<QString> allDocIds;
-    const auto resIds = m_resMgr->reservationsForBatch(m_batchId);
-    for (const auto &resId : resIds) {
-        const auto res = m_resMgr->reservation(resId);
-        const auto docIds = DocumentUtil::documentIds(res);
-        for (const auto &docId : docIds) {
-            if (m_documentMgr->hasDocument(docId.toString())) {
-                allDocIds.insert(docId.toString());
-            }
-        }
-    }
-    return allDocIds.size();
-}
-
 void TimelineDelegateController::addToCalendar(KCalendarCore::Calendar *cal)
 {
     const auto resIds = m_resMgr->reservationsForBatch(m_batchId);
@@ -893,4 +922,82 @@ void TimelineDelegateController::addToCalendar(KCalendarCore::Calendar *cal)
     }
 }
 
+static void applyVehicleLayout(KPublicTransport::Stopover &stop, const KPublicTransport::Stopover &layout)
+{
+    using namespace KPublicTransport;
+
+    if (!Stopover::isSame(stop, layout)) {
+        qDebug() << "stop mismatch";
+        return;
+    }
+
+    const auto platform = stop.hasExpectedPlatform() ? stop.expectedPlatform() : stop.scheduledPlatform();
+    if (platform != layout.scheduledPlatform()) { // TODO this might need a more sophisticated check if there's platform sections involved
+        qDebug() << "platform mismatch";
+        return;
+    }
+
+    stop = Stopover::merge(stop, layout);
+}
+
+void TimelineDelegateController::setVehicleLayout(const KPublicTransport::Stopover& stopover, bool arrival)
+{
+    auto jny = journey();
+    if (!arrival) {
+        auto dep = jny.departure();
+        applyVehicleLayout(dep, stopover);
+        jny.setDeparture(dep);
+        m_liveDataMgr->setJourney(m_batchId, jny);
+    } else {
+        auto arr = jny.arrival();
+        applyVehicleLayout(arr, stopover);
+        jny.setArrival(arr);
+        m_liveDataMgr->setJourney(m_batchId, jny);
+    }
+
+    Q_EMIT layoutChanged();
+}
+
+QString TimelineDelegateController::departurePlatformSections() const
+{
+    if (!m_resMgr) {
+        return {};
+    }
+    const auto res = m_resMgr->reservation(m_batchId);
+    return platformSectionsForCoachData(departure(), coachDataForReservation(res));
+}
+
+QString TimelineDelegateController::arrivalPlatformSections() const
+{
+    if (!m_resMgr) {
+        return {};
+    }
+    const auto res = m_resMgr->reservation(m_batchId);
+    return platformSectionsForCoachData(arrival(), coachDataForReservation(res));
+}
+
+QStringList TimelineDelegateController::documentIds() const
+{
+    if (!m_resMgr || !m_documentMgr || m_batchId.isEmpty()) {
+        return {};
+    }
+
+    QStringList result;
+    const auto resIds = m_resMgr->reservationsForBatch(m_batchId);
+    for (const auto &resId : resIds) {
+        const auto res = m_resMgr->reservation(resId);
+        const auto docIds = DocumentUtil::documentIds(res);
+        for (const auto &docId : docIds) {
+            const auto id = docId.toString();
+            if (!id.isEmpty() && m_documentMgr->hasDocument(id)) {
+                result.push_back(id);
+            }
+        }
+    }
+
+    std::sort(result.begin(), result.end());
+    result.erase(std::unique(result.begin(), result.end()), result.end());
+    return result;
+}
+
 #include "moc_timelinedelegatecontroller.cpp"
